# Observer/Observable pattern for event delegation
# Manuel Amador (Rudd-O) written on 2005-11-28
# under the GPL

import logging
logger = logging.getLogger("observable")


class Observer:
	"""An object capable of observating an Observable
	To implement an Observer, you:
		- subclass your class from Observer
		- implement the notify() method, keeping in mind that
		  the notify() method executes in the Observable's thread
		  context.  Your notify() method implementation must
		  never block, or else notifications to other objects will
		  be delayed"""
	
	def __init__(self): pass
	
	def observe(self,object):
		object.register(self)
	
	def stopObserving(self,object):
		object.unregister(self)

	def notify(self,notifierObject,eventName,*args):
		""" This is the default notify() implementation, it does
		nothing.  notify() receives the notifier object (the one
		emitting the notification), the event name (a string)
		and a variable number of positional arguments, which
		usually depend on the observable's policy. """
		pass


class Observable:
	"""The Observable object.  Observable objects allow registration
	management of observers via the register() and unregister() events.
	
	To implement an Observable object:
		- subclass your class from Observable
		- whenever you want to notify observers of a relevant event
		  use the broadcastEvent() method, passing an event name
		  and a variable number of arguments.  These will be
		  broadcast to all observing objects, in the Observable's
		  thread context."""

	def __init__(self):
		self.__registeredObservers = []

	def register(self,observerObject):
		logger.debug( "%s registering %s as observer",self,observerObject)
		if not observerObject in self.__registeredObservers:
			self.__registeredObservers.append(observerObject)

	def unregister(self,observerObject):
		logger.debug( "%s unregistering %s",self,observerObject)
		if observerObject in self.__registeredObservers:
			self.__registeredObservers.remove(observerObject)

	def broadcastEvent(self,eventName,*args):
		logger.debug( "Broadcasting event %s,%s",eventName,args)
		for o in self.__registeredObservers[:]:
			o.notify(self,eventName,*args)
# end observer/observable abstraction

__all__ = ["Observer","Observable"]